import { IPractice } from '../IPractice';
import { PracticeEvaluationResult, PracticeImpact, ProgrammingLanguage } from '../../model';
import { DxPractice } from '../DxPracticeDecorator';
import { PracticeContext } from '../../contexts/practice/PracticeContext';

@DxPractice({
  id: 'JavaScript.LoggerUsed',
  name: 'Use a JS Logging Library',
  impact: PracticeImpact.small,
  suggestion:
    'Use a logging library to avoid errors and even cyber attacks. The most widely used logging library in the JavaScript community is Winston.',
  reportOnlyOnce: true,
  url: 'https://www.npmjs.com/package/winston/',
})
export class JsLoggerUsedPractice implements IPractice {
  async isApplicable(ctx: PracticeContext): Promise<boolean> {
    return (
      ctx.projectComponent.language === ProgrammingLanguage.JavaScript || ctx.projectComponent.language === ProgrammingLanguage.TypeScript
    );
  }

  async evaluate(ctx: PracticeContext): Promise<PracticeEvaluationResult> {
    if (ctx.packageInspector) {
      if (ctx.packageInspector.hasOneOfPackages(['bunyan', 'winston', 'node-loggly', 'morgan', 'pino'])) {
        return PracticeEvaluationResult.practicing;
      } else {
        return PracticeEvaluationResult.notPracticing;
      }
    }
    return PracticeEvaluationResult.unknown;
  }
}
